@@ -13,13 +13,14 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMain.c,v 1.4 1999/04/16 01:51:19 stanton Exp $
+ * RCS: @(#) $Id: tkMain.c,v 1.15.2.2 2005/10/18 15:16:01 dgp Exp $
  */
 
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
 #include <tcl.h>
+#include <tclInt.h>
 #include <tk.h>
 #include "tkInt.h"
 #ifdef NO_STDLIB_H
@@ -30,6 +31,9 @@
 #ifdef __WIN32__
 #include "tkWinInt.h"
 #endif
+#ifdef MAC_OSX_TK
+#include "tkMacOSXInt.h"
+#endif
 
 
 typedef struct ThreadSpecificData {
@@ -42,7 +46,7 @@ typedef struct ThreadSpecificData {
 				 * terminal-like device.  Zero means it's
 				 * a file. */
 } ThreadSpecificData;
-Tcl_ThreadDataKey dataKey;
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Declarations for various library procedures and variables (don't want
@@ -54,13 +58,13 @@ Tcl_ThreadDataKey dataKey;
  */
 
 #if !defined(__WIN32__) && !defined(_WIN32)
+#if !defined(MAC_TCL)
 extern int		isatty _ANSI_ARGS_((int fd));
+#else
+#include <unistd.h>
+#endif
 extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
 #endif
-extern void		TkpDisplayWarning _ANSI_ARGS_((char *msg,
-			    char *title));
-
-extern void TkConsoleCreate_ _ANSI_ARGS_((void));
 
 /*
  * Forward declarations for procedures defined later in this file.
@@ -73,7 +77,7 @@ static void		StdinProc _ANSI_ARGS_((ClientData clientData,
 /*
  *----------------------------------------------------------------------
  *
- * TkMainEx --
+ * Tk_MainEx --
  *
  *	Main program for Wish and most other Tk-based applications.
  *
@@ -98,12 +102,11 @@ Tk_MainEx(argc, argv, appInitProc, interp)
 					 * to execute commands. */
     Tcl_Interp *interp;
 {
-    char *args, *fileName;
-    char buf[TCL_INTEGER_SPACE];
+    Tcl_Obj *argvPtr;
     int code;
     size_t length;
     Tcl_Channel inChannel, outChannel;
-    Tcl_DString argString;
+    Tcl_DString appName;
     ThreadSpecificData *tsdPtr;
 #ifdef __WIN32__
     HANDLE handle;
@@ -125,7 +128,13 @@ Tk_MainEx(argc, argv, appInitProc, interp)
     tsdPtr->interp = interp;
 
 #if (defined(__WIN32__) || defined(MAC_TCL))
-    TkConsoleCreate_();
+    Tk_InitConsoleChannels(interp);
+#endif
+
+#ifdef MAC_OSX_TK
+    if (TclGetStartupScriptFileName() == NULL) {
+        TkMacOSXDefaultStartupScript();
+    }
 #endif
     
 #ifdef TCL_MEM_DEBUG
@@ -139,7 +148,6 @@ Tk_MainEx(argc, argv, appInitProc, interp)
      * use it as the name of a script file to process.
      */
 
-    fileName = NULL;
     if (argc > 1) {
 	length = strlen(argv[1]);
 	if ((length >= 2) && (strncmp(argv[1], "-file", length) == 0)) {
@@ -147,31 +155,41 @@ Tk_MainEx(argc, argv, appInitProc, interp)
 	    argv++;
 	}
     }
-    if ((argc > 1) && (argv[1][0] != '-')) {
-	fileName = argv[1];
-	argc--;
-	argv++;
+    if (TclGetStartupScriptFileName() == NULL) {
+	if ((argc > 1) && (argv[1][0] != '-')) {
+	    TclSetStartupScriptFileName(argv[1]);
+	    argc--;
+	    argv++;
+	}
     }
-
+    
     /*
      * Make command-line arguments available in the Tcl variables "argc"
      * and "argv".
      */
 
-    args = Tcl_Merge(argc-1, argv+1);
-    Tcl_ExternalToUtfDString(NULL, args, -1, &argString);
-    Tcl_SetVar(interp, "argv", Tcl_DStringValue(&argString), TCL_GLOBAL_ONLY);
-    Tcl_DStringFree(&argString);
-    ckfree(args);
-    sprintf(buf, "%d", argc-1);
-
-    if (fileName == NULL) {
-	Tcl_ExternalToUtfDString(NULL, argv[0], -1, &argString);
+    if (TclGetStartupScriptFileName() == NULL) {
+	Tcl_ExternalToUtfDString(NULL, argv[0], -1, &appName);
     } else {
-	fileName = Tcl_ExternalToUtfDString(NULL, fileName, -1, &argString);
+	TclSetStartupScriptFileName(Tcl_ExternalToUtfDString(NULL,
+		TclGetStartupScriptFileName(), -1, &appName));
     }
-    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
-    Tcl_SetVar(interp, "argv0", Tcl_DStringValue(&argString), TCL_GLOBAL_ONLY);
+    Tcl_SetVar(interp, "argv0", Tcl_DStringValue(&appName), TCL_GLOBAL_ONLY);
+    Tcl_DStringFree(&appName);
+    argc--;
+    argv++;
+
+    Tcl_SetVar2Ex(interp, "argc", NULL, Tcl_NewIntObj(argc), TCL_GLOBAL_ONLY);
+
+    argvPtr = Tcl_NewListObj(0, NULL);
+    while (argc--) {
+	Tcl_DString ds;
+	Tcl_ExternalToUtfDString(NULL, *argv++, -1, &ds);
+	Tcl_ListObjAppendElement(NULL, argvPtr, Tcl_NewStringObj(
+		Tcl_DStringValue(&ds), Tcl_DStringLength(&ds)));
+	Tcl_DStringFree(&ds);
+    }
+    Tcl_SetVar2Ex(interp, "argv", NULL, argvPtr, TCL_GLOBAL_ONLY);
 
     /*
      * Set the "tcl_interactive" variable.
@@ -209,7 +227,8 @@ Tk_MainEx(argc, argv, appInitProc, interp)
     tsdPtr->tty = isatty(0);
 #endif
     Tcl_SetVar(interp, "tcl_interactive",
-	    ((fileName == NULL) && tsdPtr->tty) ? "1" : "0", TCL_GLOBAL_ONLY);
+	    ((TclGetStartupScriptFileName() == NULL) 
+		    && tsdPtr->tty) ? "1" : "0", TCL_GLOBAL_ONLY);
 
     /*
      * Invoke application-specific initialization.
@@ -224,9 +243,9 @@ Tk_MainEx(argc, argv, appInitProc, interp)
      * Invoke the script specified on the command line, if any.
      */
 
-    if (fileName != NULL) {
+    if (TclGetStartupScriptFileName() != NULL) {
 	Tcl_ResetResult(interp);
-	code = Tcl_EvalFile(interp, fileName);
+	code = Tcl_EvalFile(interp, TclGetStartupScriptFileName());
 	if (code != TCL_OK) {
 	    /*
 	     * The following statement guarantees that the errorInfo
@@ -261,7 +280,6 @@ Tk_MainEx(argc, argv, appInitProc, interp)
 	    Prompt(interp, 0);
 	}
     }
-    Tcl_DStringFree(&argString);
 
     outChannel = Tcl_GetStdChannel(TCL_STDOUT);
     if (outChannel) {
@@ -401,12 +419,12 @@ Prompt(interp, partial)
 					 * exists a partial command, so use
 					 * the secondary prompt. */
 {
-    char *promptCmd;
+    Tcl_Obj *promptCmd;
     int code;
     Tcl_Channel outChannel, errChannel;
 
-    promptCmd = Tcl_GetVar(interp,
-	partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
+    promptCmd = Tcl_GetVar2Ex(interp,
+	partial ? "tcl_prompt2" : "tcl_prompt1", NULL, TCL_GLOBAL_ONLY);
     if (promptCmd == NULL) {
 defaultPrompt:
 	if (!partial) {
@@ -423,7 +441,7 @@ defaultPrompt:
             }
 	}
     } else {
-	code = Tcl_Eval(interp, promptCmd);
+	code = Tcl_EvalObjEx(interp, promptCmd, TCL_EVAL_GLOBAL);
 	if (code != TCL_OK) {
 	    Tcl_AddErrorInfo(interp,
 		    "\n    (script that generates prompt)");
