@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkScrollbar.h 1.8 96/11/05 11:34:58
+ * RCS: @(#) $Id: tkScrollbar.h,v 1.6 2000/11/22 01:49:38 ericm Exp $
  */
 
 #ifndef _TKSCROLLBAR
@@ -135,9 +135,6 @@ typedef struct TkScrollbar {
 				 * scripts.  Malloc'ed, but may be NULL. */
     int flags;			/* Various flags;  see below for
 				 * definitions. */
-    Tk_Tile tile, activeTile, troughTile;
-    GC activeTileGC;
-    Tk_TSOffset tsoffset;
 } TkScrollbar;
 
 /*
@@ -174,7 +171,7 @@ typedef struct TkScrollbar {
  * Declaration of scrollbar class procedures structure.
  */
 
-extern TkClassProcs tkpScrollbarProcs;
+extern Tk_ClassProcs tkpScrollbarProcs;
 
 /*
  * Declaration of scrollbar configuration options.
