@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- *  RCS: @(#) $Id: tkMacOSXEmbed.c,v 1.2 2002/08/31 06:12:29 das Exp $
+ *  RCS: @(#) $Id: tkMacOSXEmbed.c,v 1.2.2.1 2005/08/09 07:39:54 das Exp $
  */
 
 #include "tkInt.h"
@@ -242,7 +242,8 @@ TkpUseWindow(
     int result;
 
     if (winPtr->window != None) {
-        panic("TkpUseWindow: X window already assigned");
+	Tcl_AppendResult(interp, "can't modify container after widget is created", (char *) NULL);
+	return TCL_ERROR;
     }
     
     /*
@@ -502,7 +503,7 @@ TkMacOSXContainerId(winPtr)
             return (MacDrawable *) containerPtr->parent;
         }
     }
-    panic("TkMacOSXContainerId couldn't find window");
+    Tcl_Panic("TkMacOSXContainerId couldn't find window");
     return None;
 }
 
@@ -831,7 +832,7 @@ ContainerEventProc(clientData, eventPtr)
             containerPtr->parent != eventPtr->xmaprequest.parent;
             containerPtr = containerPtr->nextPtr) {
         if (containerPtr == NULL) {
-            panic("ContainerEventProc couldn't find Container record");
+            Tcl_Panic("ContainerEventProc couldn't find Container record");
         }
     }
 
