@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXFont.c,v 1.3.2.2 2004/11/12 09:03:40 das Exp $
+ * RCS: @(#) $Id: tkMacOSXFont.c,v 1.3.2.5 2005/11/27 02:36:46 das Exp $
  */
 #include <Carbon/Carbon.h>
 
@@ -87,7 +87,7 @@ static TkStateMap scriptMap[] = {
     {smEastEurRoman,    "macCentEuro"},
     {smVietnamese,      "macVietnam"},
     {smExtArabic,       "macSindhi"},
-    {NULL,               NULL}
+    {0,                 NULL}
 };    
 
 static TkStateMap romanMap[] = {
@@ -97,13 +97,13 @@ static TkStateMap romanMap[] = {
     {langRomanian,        "macRomania"},
     {langTurkish,         "macTurkish"},
     {langGreek,           "macGreek"},
-    {NULL,                NULL}
+    {0,                   NULL}
 };
 
 static TkStateMap cyrillicMap[] = {
     {langUkrainian,        "macUkraine"},
     {langBulgarian,        "macBulgaria"},
-    {NULL,                NULL}
+    {0,                    NULL}
 };
 
 /*
@@ -255,6 +255,8 @@ static int GetFamilyOrAliasNum(CONST char *faceName, short *familyPtr);
 static Tcl_Encoding GetFontEncoding(int faceNum, int allowSymbol, int *isSymbolPtr);
 static Tk_Uid GetUtfFaceName(StringPtr faceNameStr);
 
+void TkMacOSXInitControlFontStyle(Tk_Font tkfont,
+	ControlFontStylePtr fsPtr);
 
 /*
  *-------------------------------------------------------------------------
@@ -294,7 +296,7 @@ TkpFontPkgInit(mainPtr)
          * Used for saving and restoring state while drawing and measuring.
          */
         if (NewGWorld(&gWorld, 0, &rect, NULL, NULL, 0) != noErr) {
-            panic("TkpFontPkgInit: NewGWorld failed");
+            Tcl_Panic("TkpFontPkgInit: NewGWorld failed");
         }
         /*
          * The name of each font is stored in the encoding of that font.
@@ -361,7 +363,7 @@ TkpFontPkgInit(mainPtr)
             }
         }
         if (fontMapOffset != numFonts) {
-            panic("TkpFontPkgInit: unexpected number of fonts");
+            Tcl_Panic("TkpFontPkgInit: unexpected number of fonts");
         }
 
         mapPtr = &newFontNameMap[numFonts];
@@ -2113,7 +2115,7 @@ GetUtfFaceName(
             return mapPtr->utfName;
         }
     }
-    panic("GetUtfFaceName: unexpected nativeName");
+    Tcl_Panic("GetUtfFaceName: unexpected nativeName");
     return NULL;
 }
 
@@ -2239,8 +2241,6 @@ TkMacOSXInitControlFontStyle(Tk_Font tkfont, ControlFontStylePtr fsPtr)
  *----------------------------------------------------------------------
  */
 
-#include <mach-o/dyld.h>
-
 /* define constants from 10.2 Quickdraw.h to enable compilation in 10.1 */
 #define kQDUseTrueTypeScalerGlyphs      (1 << 0)
 #define kQDUseCGTextRendering           (1 << 1)
@@ -2260,7 +2260,8 @@ TkMacOSXAntialiasedTextVariableProc(clientData, interp, name1, name2, flags)
     return (char *) NULL;
 }
 
-int TkMacOSXUseAntialiasedText(interp, enable)
+int
+TkMacOSXUseAntialiasedText(interp, enable)
         Tcl_Interp *interp;
         int enable;
 {
@@ -2268,14 +2269,9 @@ int TkMacOSXUseAntialiasedText(interp, enable)
     static UInt32 (*swaptextflags)(UInt32) = NULL;
     
     if(!initialized) {
-        NSSymbol nsSymbol = NULL;
-        if(NSIsSymbolNameDefinedWithHint("_QDSwapTextFlags", "QD")) {
-            nsSymbol = NSLookupAndBindSymbolWithHint("_QDSwapTextFlags", "QD");
-        } else if(NSIsSymbolNameDefinedWithHint("_SwapQDTextFlags", "QD")) {
-            nsSymbol = NSLookupAndBindSymbolWithHint("_SwapQDTextFlags", "QD");
-        }
-        if(nsSymbol) {
-            swaptextflags = NSAddressOfSymbol(nsSymbol);
+        swaptextflags = TkMacOSXGetNamedSymbol("QD", "_QDSwapTextFlags");
+        if (!swaptextflags) {
+            swaptextflags = TkMacOSXGetNamedSymbol("QD", "_SwapQDTextFlags");
         }
         initialized = TRUE;
 
