@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXMenubutton.c,v 1.2.2.1 2004/02/16 00:42:34 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXMenubutton.c,v 1.2.2.6 2005/11/27 06:47:33 das Exp $
  */
 
 #include <Carbon/Carbon.h>
@@ -19,6 +19,12 @@
 #include "tkMacOSXInt.h"
 #include "tkMacOSXDebug.h"
 
+#if !defined(MAC_OS_X_VERSION_10_3) || \
+        (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3)
+    /* Define constants only available on Mac OS X 10.3 or later */
+    #define kMenuAttrDoNotUseUserCommandKeys (1 << 7)
+#endif
+
 #define kShadowOffset   (3)     /* amount to offset shadow from frame */
 #define kTriangleWidth  (11)    /* width of the triangle */
 #define kTriangleHeight (6)     /* height of the triangle */
@@ -87,8 +93,6 @@ static void CompareControlTitleParams(
 extern int TkFontGetFirstTextLayout(Tk_TextLayout layout, Tk_Font * font, char * dst); 
 extern void TkMacOSXInitControlFontStyle(Tk_Font tkfont,ControlFontStylePtr fsPtr);
 
-extern int tkPictureIsOpen;
-
 /*
  * The structure below defines menubutton class behavior by means of
  * procedures that can be invoked from generic window code.
@@ -214,7 +218,9 @@ TkpDisplayMenuButton(
      }
      if (!mbPtr->userPane) {
          if (MenuButtonInitControl(mbPtr,&paneRect,&cntrRect ) ) {
+#ifdef TK_MAC_DEBUG
              fprintf(stderr,"Init Control failed\n" );
+#endif
              return;
          }
      }
@@ -234,7 +240,7 @@ TkpDisplayMenuButton(
         if (titleChanged) {
             CFStringRef cf;    	    
             cf = CFStringCreateWithCString(NULL,
-                  titleParams.title, kCFStringEncodingUTF8);
+                  (char*) titleParams.title, kCFStringEncodingUTF8);
             if (hasImageOrBitmap) {
                 SetControlTitleWithCFString(mbPtr->control, cf);
             } else {
@@ -248,7 +254,9 @@ TkpDisplayMenuButton(
             if (hasImageOrBitmap) {
 	      err = SetControlFontStyle(mbPtr->control,&titleParams.style);
                 if (err !=noErr) {
+#ifdef TK_MAC_DEBUG
                     fprintf(stderr,"SetControlFontStyle failed %d\n", err);
+#endif
                     return;
                 }
             }
@@ -268,7 +276,9 @@ TkpDisplayMenuButton(
         /* Set the flag to circumvent clipping and bounds problems with OS 10.0.4 */
         tkPictureIsOpen = 1;
         if (!(mbPtr->bevelButtonContent.u.picture = OpenCPicture(&mbPtr->picParams)) ) {
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"OpenCPicture failed\n");
+#endif
         }
         /*
          * TO DO - There is one case where XCopyPlane calls CopyDeepMask,
@@ -290,7 +300,9 @@ TkpDisplayMenuButton(
                     sizeof(ControlButtonContentInfo),
 			   (char *) &mbPtr->bevelButtonContent);
         if (err != noErr) {
+#ifdef TK_MAC_DEBUG
                 fprintf(stderr,"SetControlData BevelButtonContent failed, %d\n", err );
+#endif
         }
         switch (butPtr->anchor) {
             case TK_ANCHOR_N:
@@ -327,7 +339,9 @@ TkpDisplayMenuButton(
                 sizeof(ControlButtonGraphicAlignment),
 		(char *) &theAlignment);
         if (err != noErr ) {
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"SetControlData BevelButtonGraphicAlign failed, %d\n", err );
+#endif
         }
     }
     if (butPtr->flags & GOT_FOCUS) {
@@ -559,7 +573,7 @@ static void
 ComputeControlTitleParams(TkMenuButton * butPtr, ControlTitleParams * paramsPtr )
 {
     Tk_Font font;
-    paramsPtr->len =TkFontGetFirstTextLayout(butPtr->textLayout,&font, paramsPtr->title);
+    paramsPtr->len =TkFontGetFirstTextLayout(butPtr->textLayout,&font, (char*) paramsPtr->title);
     paramsPtr->title [paramsPtr->len] = 0;
     if (paramsPtr->len) {
         TkMacOSXInitControlFontStyle(font,&paramsPtr->style);
@@ -593,7 +607,7 @@ MenuButtonInitControl (
     Rect      *cntrRect
 )
 {
-    OSErr      status;
+    OSStatus   err;
     TkMenuButton * butPtr = ( TkMenuButton * )mbPtr;
     ControlRef rootControl;
     SInt16     procID;
@@ -602,10 +616,7 @@ MenuButtonInitControl (
     SInt16     minValue;
     SInt16     maxValue;
     SInt32     controlReference;
-    int        err;
     short      menuID;
-    int        length;
-    Str255     itemText;
 
     rootControl = TkMacOSXGetRootControl(Tk_WindowId(butPtr->tkwin));
     mbPtr->windowRef = GetWindowFromPort(TkMacOSXGetDrawablePort(Tk_WindowId(butPtr->tkwin)));
@@ -628,12 +639,16 @@ MenuButtonInitControl (
             procID,
             controlReference );
     if (!mbPtr->userPane) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"Failed to create user pane control\n");
+#endif
         return 1;
     }
-    status = EmbedControl(mbPtr->userPane,rootControl);
-    if (status != noErr) {
-        fprintf(stderr,"Failed to embed user pane control %d\n", status);
+    err = EmbedControl(mbPtr->userPane,rootControl);
+    if (err != noErr) {
+#ifdef TK_MAC_DEBUG
+        fprintf(stderr,"Failed to embed user pane control %d\n", (int) err);
+#endif
         return 1;
     }
     SetUserPaneSetUpSpecialBackgroundProc(mbPtr->userPane,
@@ -652,46 +667,69 @@ MenuButtonInitControl (
         mbPtr->params.procID,
         controlReference );
     if (!mbPtr->control) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"failed to create control of type %d : line %d\n",mbPtr->params.procID, __LINE__);
+#endif
         return 1;
     }
     err = EmbedControl(mbPtr->control,mbPtr->userPane);
     if (err != noErr ) {
-        fprintf(stderr,"failed to embed control of type %d,%d\n",procID, err);
+#ifdef TK_MAC_DEBUG
+        fprintf(stderr,"failed to embed control of type %d,%d\n",procID, (int) err);
+#endif
         return 1;
     }
     if (mbPtr->params.isBevel) {
             CFStringRef cf;    	    
             cf = CFStringCreateWithCString(NULL,
-                  mbPtr->titleParams.title, kCFStringEncodingUTF8);
+                  (char*) mbPtr->titleParams.title, kCFStringEncodingUTF8);
         SetControlTitleWithCFString(mbPtr->control, cf);
         CFRelease(cf);
         if (mbPtr->titleParams.len) {
 	  err = SetControlFontStyle(mbPtr->control,&mbPtr->titleParams.style);
             if (err !=noErr) {
-                fprintf(stderr,"SetControlFontStyle failed %d\n", err);
+#ifdef TK_MAC_DEBUG
+                fprintf(stderr,"SetControlFontStyle failed %d\n", (int) err);
+#endif
                 return 1;
              }
         }
     } else {
-        CFStringRef cf;    	    
+        CFStringRef cfStr;
         err = TkMacOSXGetNewMenuID(mbPtr->info.interp, (TkMenu *)mbPtr, 0, &menuID);
         if (err != TCL_OK) {
-            return err;
+            return 1;
         }       
-        length = strlen(Tk_PathName(mbPtr->info.tkwin));
-        memmove(&itemText[1], Tk_PathName(mbPtr->info.tkwin),
-                (length > 230) ? 230 : length);
-        itemText[0] = (length > 230) ? 230 : length;
-        if (!(mbPtr->menuRef = NewMenu(menuID,itemText))) {
+        err = CreateNewMenu(menuID, kMenuAttrDoNotUseUserCommandKeys,
+                &(mbPtr->menuRef));
+        if (err != noErr) {
+#ifdef TK_MAC_DEBUG
+            fprintf(stderr,"CreateNewMenu failed, %d.\n", (int) err);
+#endif
             return 1;
         }
-        cf = CFStringCreateWithCString(NULL,
-                mbPtr->titleParams.title, kCFStringEncodingUTF8);
+        cfStr = CFStringCreateWithCString(NULL, Tk_PathName(mbPtr->info.tkwin),
+                kCFStringEncodingUTF8);
+        if (!cfStr) {
+#ifdef TK_MAC_DEBUG
+            fprintf(stderr,"CFStringCreateWithCString failed.\n");
+#endif
+            return 1;
+        }
+        err = SetMenuTitleWithCFString(mbPtr->menuRef, cfStr);
+        CFRelease(cfStr);
+        if (err != noErr) {
+#ifdef TK_MAC_DEBUG
+            fprintf(stderr,"SetMenuTitleWithCFString failed, %d.\n", (int) err);
+#endif
+            return 1;
+        }
+        cfStr = CFStringCreateWithCString(NULL,
+                (char*) mbPtr->titleParams.title, kCFStringEncodingUTF8);
         AppendMenuItemText(mbPtr->menuRef, "\px");
-        if (cf != NULL) {
-        SetMenuItemTextWithCFString(mbPtr->menuRef, 1, cf);
-        CFRelease(cf);
+        if (cfStr) {
+            SetMenuItemTextWithCFString(mbPtr->menuRef, 1, cfStr);
+            CFRelease(cfStr);
         }
         err = SetControlData(mbPtr->control,
                 kControlNoPart,
