@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXMenus.c,v 1.2.2.3 2004/11/11 01:26:43 das Exp $
+ * RCS: @(#) $Id: tkMacOSXMenus.c,v 1.2.2.6 2005/11/27 06:53:36 das Exp $
  */
 
 #include "tk.h"
@@ -24,6 +24,12 @@
 #undef Status
 #include <Carbon/Carbon.h>
 
+#if !defined(MAC_OS_X_VERSION_10_3) || \
+        (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3)
+    /* Define constants only available on Mac OS X 10.3 or later */
+    #define kMenuAttrDoNotUseUserCommandKeys (1 << 7)
+#endif
+
 #define kAppleMenu              256
 #define kAppleAboutItem         1
 #define kFileMenu               2
@@ -64,16 +70,15 @@ static void SourceDialog _ANSI_ARGS_((void));
 
 void 
 TkMacOSXHandleMenuSelect(
-    long mResult,
+    MenuID theMenu,
+    MenuItemIndex theItem,
     int optionKeyPressed)
 {
-    short theItem = LoWord(mResult);
-    short theMenu = HiWord(mResult);
     Tk_Window tkwin;
     Window window;
     TkDisplay *dispPtr;
 
-    if (mResult == 0) {
+    if (theItem == 0) {
         TkMacOSXHandleTearoffMenu();
         TkMacOSXClearMenubarActive();
         return;
@@ -104,7 +109,7 @@ TkMacOSXHandleMenuSelect(
                     break;
                 case kCloseItem:
                     /* Send close event */
-                    window = TkMacOSXGetXWindow(FrontNonFloatingWindow());
+                    window = TkMacOSXGetXWindow(ActiveNonFloatingWindow());
                     dispPtr = TkGetDisplayList();
                     tkwin = Tk_IdToWindow(dispPtr->display, window);
                     TkGenWMDestroyEvent(tkwin);
@@ -150,52 +155,51 @@ void
 TkMacOSXInitMenus(
     Tcl_Interp *interp)
 {
+    OSStatus err;
     gInterp = interp;
 
-    /* 
-     * At this point, InitMenus() should have already been called. 
-     */
-
-    if (TkMacOSXUseMenuID(256) != TCL_OK) {
-            panic("Menu ID 256 is already in use!");
+    if (TkMacOSXUseMenuID(kAppleMenu) != TCL_OK) {
+	Tcl_Panic("Menu ID %d is already in use!", kAppleMenu);
     }
-    tkAppleMenu = NewMenu(256, "\p\024");
-    if (tkAppleMenu == NULL) {
-        panic("memory - menus");
+    err = CreateNewMenu(kAppleMenu, kMenuAttrDoNotUseUserCommandKeys,
+            &tkAppleMenu);
+    if (err != noErr) {
+        Tcl_Panic("CreateNewMenu failed !");
     }
+    SetMenuTitle(tkAppleMenu, "\p\024");
     InsertMenu(tkAppleMenu, 0);
     AppendMenu(tkAppleMenu, "\pAbout Tcl & Tk");
     AppendMenu(tkAppleMenu, "\p(-");
-    /* Not necessary in Carbon:
-    AppendResMenu(tkAppleMenu, 'DRVR');
-    */
 
     if (TkMacOSXUseMenuID(kFileMenu) != TCL_OK) {
-            panic("Menu ID %d is already in use!", kFileMenu);
+	Tcl_Panic("Menu ID %d is already in use!", kFileMenu);
     }
-    tkFileMenu = NewMenu(kFileMenu, "\pFile");
-    if (tkFileMenu == NULL) {
-        panic("memory - menus");
+    err = CreateNewMenu(kFileMenu, kMenuAttrDoNotUseUserCommandKeys,
+            &tkFileMenu);
+    if (err != noErr) {
+        Tcl_Panic("CreateNewMenu failed !");
     }
+    SetMenuTitle(tkFileMenu, "\pFile");
     InsertMenu(tkFileMenu, 0);
     AppendMenu(tkFileMenu, "\pSource");
     AppendMenu(tkFileMenu, "\pClose/W");
 
-
     if (TkMacOSXUseMenuID(kEditMenu) != TCL_OK) {
-            panic("Menu ID %d is already in use!", kEditMenu);
+	Tcl_Panic("Menu ID %d is already in use!", kEditMenu);
     }
-    tkEditMenu = NewMenu(kEditMenu, "\pEdit");
-    if (tkEditMenu == NULL) {
-        panic("memory - menus");
+    err = CreateNewMenu(kEditMenu, kMenuAttrDoNotUseUserCommandKeys,
+            &tkEditMenu);
+    if (err != noErr) {
+        Tcl_Panic("CreateNewMenu failed !");
     }
+    SetMenuTitle(tkEditMenu, "\pEdit");
     InsertMenu(tkEditMenu, 0);
     AppendMenu(tkEditMenu, "\pCut/X");
     AppendMenu(tkEditMenu, "\pCopy/C");
     AppendMenu(tkEditMenu, "\pPaste/V");
     AppendMenu(tkEditMenu, "\pClear");
     if (TkMacOSXUseMenuID(kHMHelpMenuID) != TCL_OK) {
-            panic("Help menu ID %s is already in use!", kHMHelpMenuID);
+	Tcl_Panic("Help menu ID %s is already in use!", kHMHelpMenuID);
     }
     
     DrawMenuBar();
@@ -229,7 +233,7 @@ GenerateEditEvent(
     Window window;
     TkDisplay *dispPtr;
 
-    window = TkMacOSXGetXWindow(FrontNonFloatingWindow());
+    window = TkMacOSXGetXWindow(ActiveNonFloatingWindow());
     dispPtr = TkGetDisplayList();
     tkwin = Tk_IdToWindow(dispPtr->display, window);
     tkwin = (Tk_Window) ((TkWindow *) tkwin)->dispPtr->focusPtr;
@@ -246,15 +250,15 @@ GenerateEditEvent(
     event.subwindow = None;
     event.time = TkpGetMS();
     
-    GetMouse(&where);
+    XQueryPointer(NULL, None, NULL, NULL,
+            &event.x_root, &event.y_root, NULL, NULL, &event.state);
+    where.h = event.x_root;
+    where.v = event.y_root;
+    GlobalToLocal(&where);
     tkwin = Tk_TopCoordsToWindow(tkwin, where.h, where.v, 
             &event.x, &event.y);
-    LocalToGlobal(&where);
-    event.x_root = where.h;
-    event.y_root = where.v;
-    event.state = TkMacOSXButtonKeyState();
     event.same_screen = true;
-    
+
     switch (flag) {
         case EDIT_CUT:
             event.name = Tk_GetUid("Cut");
