@@ -11,11 +11,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5.2.2 2004/02/16 00:42:34 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5.2.6 2005/10/18 00:41:19 hobbs Exp $
  */
 
 #include "tkScrollbar.h"
 #include "tkMacOSXInt.h"
+#include "tclInt.h"
 
 #include <Carbon/Carbon.h>
 
@@ -97,9 +98,11 @@ static ControlActionUPP scrollActionProc = NULL; /* Pointer to func. */
 static ThumbActionUPP thumbActionProc = NULL;    /* Pointer to func. */
 static TkScrollbar *activeScrollPtr = NULL;      /* Non-null when in thumb */
                                                  /* proc. */
-static Point activePoint;                        /* Used when
-                                                  * activeScrollPtr is
-                                                  * non-NULL */
+static Point mouseDownPoint;	/* Used when activeScrollPtr is non-NULL   */
+				/* to store the coordinates where the      */
+				/* mouse was first pressed to begin        */
+				/* dragging the thumb, because             */
+				/* ThumbActionProc can't take any args.    */
 						 
 /*
  * Forward declarations for procedures defined later in this file:
@@ -266,7 +269,7 @@ TkpDisplayScrollbar(
          * If we are foremost then make us active.
          */
         
-        frontNonFloating = FrontNonFloatingWindow();
+        frontNonFloating = ActiveNonFloatingWindow();
         
         if ((windowRef == FrontWindow()) || TkpIsWindowFloating(windowRef)) {
             macScrollPtr->macFlags |= ACTIVE;
@@ -583,7 +586,7 @@ ThumbActionProc()
     register MacScrollbar *macScrollPtr = (MacScrollbar *) activeScrollPtr;
     Tcl_DString cmdString;
     int origValue;
-    double thumbWidth, newFirstFraction, trackBarSize;
+    double thumbWidth, oldFirstFraction, newFirstFraction, trackBarSize;
     char valueString[40];
     Point currentPoint = { 0, 0 };
     Rect trackRect;
@@ -626,9 +629,16 @@ ThumbActionProc()
     /*
      * Track the mouse while the button is held down.  If the mouse is moved,
      * we calculate the value that should be passed to the "command" part of
-     * the scrollbar.
+     * the scrollbar.  Since the mouse may move a distance too small to
+     * cause a change to the first fraction, each calculation must be done
+     * versus what the first fraction was when the mouse button was
+     * initially pressed.  Otherwise, moving the mouse too slowly will
+     * cause the calculated fraction delta to be zero and the scrollbar
+     * won't respond.
      */
 
+    oldFirstFraction = scrollPtr->firstFraction;
+
     do {
         err = TrackMouseLocationWithOptions(NULL,
                 kTrackMouseLocationOptionDontConsumeMouseUp,
@@ -640,18 +650,25 @@ ThumbActionProc()
         if ((err == noErr) 
                 && ((trackingResult == kMouseTrackingMouseDragged)
                 || (trackingResult == kMouseTrackingMouseMoved))) {
-        /*
-         * Calculate where the scrollbar should move to, and reset the
-         * activePoint to where we are now.
-         */
-	    newFirstFraction = scrollPtr->firstFraction;
+
+	   /*
+            * Calculate where the scrollbar should move to, based on
+            * where the mouse button was pressed and where the scrollbar
+	    * initially was at that time.  Note that PtInRect() will
+	    * return false if currentPoint or trackRect are not in
+	    * is not in current graphics port, which may happen if any
+            * of the waiting idle events change the port (e.g. with
+	    * SetPort()) but fail to restore it before returning and the
+	    * scrollbar will lock in place.
+            */
+	    newFirstFraction = oldFirstFraction;
             if (PtInRect(currentPoint, &trackRect)) {
 		double pixDiff;
 		double fracDelta;
                 if (scrollPtr->vertical == true) {
-		    pixDiff = (double)(currentPoint.v - activePoint.v);
+		    pixDiff = (double)(currentPoint.v - mouseDownPoint.v);
                 } else {
-		    pixDiff = (double)(currentPoint.h - activePoint.h);
+		    pixDiff = (double)(currentPoint.h - mouseDownPoint.h);
                 }
 		fracDelta = pixDiff/(trackBarSize);
 		newFirstFraction += fracDelta;
@@ -662,8 +679,13 @@ ThumbActionProc()
 		}
 	    }
             
-            activePoint = currentPoint;
-            
+	    /*
+	     * Move the scrollbar thumb to the new first fraction given
+	     * its position when initially pressed and how far the mouse
+	     * has moved.  Process waiting idle tasks afterward to allow
+	     * for the display to update.
+	     */
+
             sprintf(valueString, "%g", newFirstFraction);
             Tcl_DStringSetLength(&cmdString, 0);
             Tcl_DStringAppend(&cmdString, scrollPtr->command,
@@ -674,12 +696,8 @@ ThumbActionProc()
             Tcl_Preserve((ClientData) interp);
             Tcl_GlobalEval(interp, cmdString.string);
             Tcl_Release((ClientData) interp);
-            Tcl_DStringSetLength(&cmdString, 0);
-            Tcl_DStringAppend(&cmdString, "update idletasks",
-                strlen("update idletasks"));
-            Tcl_Preserve((ClientData) interp);
-            Tcl_GlobalEval(interp, cmdString.string);
-            Tcl_Release((ClientData) interp);
+
+	    TclServiceIdle();
         }
     } while ((err == noErr) && trackingResult != kMouseTrackingMouseReleased);
 
@@ -802,8 +820,8 @@ ScrollbarBindProc(
              * so the callback may have access to it.
              */
             activeScrollPtr = scrollPtr;
-	    activePoint.h = where.h;
-	    activePoint.v = where.v;
+	    mouseDownPoint.h = where.h;
+	    mouseDownPoint.v = where.v;
             part = TrackControl(macScrollPtr->sbHandle, where,
                     (ControlActionUPP) thumbActionProc);
             activeScrollPtr = NULL;
@@ -859,7 +877,6 @@ ScrollbarBindProc(
          * The TrackControl call will "eat" the ButtonUp event.  We now
          * generate a ButtonUp event so Tk will unset implicit grabs etc.
          */
-        GetMouse(&where);
         XQueryPointer(NULL, None, &window, &window, &x,
             &y, &dummy, &dummy, &state);
         window = Tk_WindowId(scrollPtr->tkwin);
@@ -959,14 +976,14 @@ UpdateControlValues(
      * NOTE: changing the control record directly may not work when
      * Apple releases the Copland version of the MacOS (or when hell is cold).
      */
-     
+
     contrlRect.left   = macDraw->xOff + scrollPtr->inset;
     contrlRect.top    = macDraw->yOff + scrollPtr->inset;
     contrlRect.right  = macDraw->xOff + Tk_Width(tkwin) - scrollPtr->inset;
     contrlRect.bottom = macDraw->yOff + Tk_Height(tkwin) - scrollPtr->inset;
-        
+
     SetControlBounds(macScrollPtr->sbHandle, &contrlRect );
-    
+
     /*
      * To make Tk applications look more like Macintosh applications without 
      * requiring additional work by the Tk developer we do some cute tricks.
@@ -976,7 +993,8 @@ UpdateControlValues(
      * it covers the area of the grow region ao the scrollbar can also draw
      * the grow region if need be.
      */
-    if (!strcmp(macDraw->winPtr->geomMgrPtr->name, "place")) {
+    if ((macDraw->winPtr->geomMgrPtr != NULL)
+	    && !strcmp(macDraw->winPtr->geomMgrPtr->name, "place")) {
         macScrollPtr->macFlags &= ~AUTO_ADJUST;
     } else {
         macScrollPtr->macFlags |= AUTO_ADJUST;
@@ -999,7 +1017,7 @@ UpdateControlValues(
             TkpComputeScrollbarGeometry(scrollPtr);
         }
     }
-    
+
     if (portRect.top == contrlRect.top) {
         if (macScrollPtr->macFlags & AUTO_ADJUST) {
             contrlRect.top--;
@@ -1016,7 +1034,7 @@ UpdateControlValues(
             TkpComputeScrollbarGeometry(scrollPtr);
         }
     }
-        
+
     if (portRect.right == contrlRect.right) {
         flushRight = true;
         if (macScrollPtr->macFlags & AUTO_ADJUST) {
@@ -1034,7 +1052,7 @@ UpdateControlValues(
             TkpComputeScrollbarGeometry(scrollPtr);
         }
     }
-        
+
     if (portRect.bottom == contrlRect.bottom) {
         flushBottom = true;
         if (macScrollPtr->macFlags & AUTO_ADJUST) {
@@ -1073,7 +1091,7 @@ UpdateControlValues(
     } else {
         TkMacOSXSetScrollbarGrow((TkWindow *) tkwin, false);
     }
-    
+
     /*
      * Given the Tk parameters for the fractions of the start and
      * end of the thumb, the following calculation determines the
@@ -1085,32 +1103,29 @@ UpdateControlValues(
      * the content area less the size of the view area.
      * Since these values are all integers, and Tk gives the thumb position
      * as fractions, we have introduced a scaling factor.
-     *   
      */
     dViewSize =  (scrollPtr->lastFraction - scrollPtr->firstFraction)
             * SCROLLBAR_SCALING_DVALUE;
-    
+
     viewSize = (SInt32) dViewSize;
     controlMax = (SInt32) (SCROLLBAR_SCALING_DVALUE - dViewSize);
     controlValue = (SInt32) (SCROLLBAR_SCALING_DVALUE * scrollPtr->firstFraction);
 
     SetControlViewSize(macScrollPtr->sbHandle,viewSize);
-    SetControl32BitValue(macScrollPtr->sbHandle, controlValue);
     SetControl32BitMaximum(macScrollPtr->sbHandle, controlMax);
 
-#if 0             
+#if 0
     middle = scrollPtr->firstFraction /
 	    (1.0 - (scrollPtr->lastFraction - scrollPtr->firstFraction));
-    
+
     viewSize = (SInt32) ((scrollPtr->lastFraction - scrollPtr->firstFraction) 
             * SCROLLBAR_SCALING_DVALUE
 	    / (1.0 - (scrollPtr->lastFraction - scrollPtr->firstFraction)));
 
-  
     SetControlViewSize(macScrollPtr->sbHandle,viewSize);
-    SetControl32BitValue(macScrollPtr->sbHandle, 
+    SetControl32BitValue(macScrollPtr->sbHandle,
 			 (middle) );
-#endif  
+#endif
     contrlHilite = GetControlHilite(macScrollPtr->sbHandle);
     SetControl32BitMinimum(macScrollPtr->sbHandle, 0);
     if ( contrlHilite == 0 || contrlHilite == 255) {
@@ -1121,7 +1136,8 @@ UpdateControlValues(
             HiliteControl(macScrollPtr->sbHandle, 0);
         }
     }
-    
+    SetControl32BitValue(macScrollPtr->sbHandle, controlValue);
+
     if ( !IsControlVisible (macScrollPtr -> sbHandle) ) {
         SetControlVisibility(macScrollPtr->sbHandle,TRUE,FALSE);
     }
