@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXXStubs.c,v 1.2.2.4 2004/11/13 03:42:10 das Exp $
+ * RCS: @(#) $Id: tkMacOSXXStubs.c,v 1.2.2.10 2005/11/30 01:02:55 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -61,16 +61,56 @@ static int DefaultErrorHandler _ANSI_ARGS_((Display* display,
  * Other declarations
  */
 
-int TkMacOSXXDestroyImage _ANSI_ARGS_((XImage *image));
-unsigned long TkMacOSXXGetPixel _ANSI_ARGS_((XImage *image, int x, int y));
-int TkMacOSXXPutPixel _ANSI_ARGS_((XImage *image, int x, int y,
+static int TkMacOSXXDestroyImage _ANSI_ARGS_((XImage *image));
+static unsigned long TkMacOSXXGetPixel _ANSI_ARGS_((XImage *image, int x, int y));
+static int TkMacOSXXPutPixel _ANSI_ARGS_((XImage *image, int x, int y,
 	unsigned long pixel));
-XImage *TkMacOSXXSubImage _ANSI_ARGS_((XImage *image, int x, int y, 
+static XImage *TkMacOSXXSubImage _ANSI_ARGS_((XImage *image, int x, int y, 
 	unsigned int width, unsigned int height));
-int TkMacOSXXAddPixel _ANSI_ARGS_((XImage *image, long value));
+static int TkMacOSXXAddPixel _ANSI_ARGS_((XImage *image, long value));
 int _XInitImageFuncPtrs _ANSI_ARGS_((XImage *image));
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXDisplayChanged --
+ *
+ *	Called to set up initial screen info or when an event indicated
+ *	display (screen) change.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	May change info regarding the screen.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkMacOSXDisplayChanged(Display *display)
+{
+    GDHandle graphicsDevice;
+    Screen *screen;
+
+    if (display == NULL || display->screens == NULL) {
+	return;
+    }
+    screen = display->screens;
+
+    graphicsDevice = GetMainDevice();
+    screen->root_depth  = (*(*graphicsDevice)->gdPMap)->cmpSize *
+                               (*(*graphicsDevice)->gdPMap)->cmpCount;
+    screen->height      = (*graphicsDevice)->gdRect.bottom -
+	(*graphicsDevice)->gdRect.top;
+    screen->width       = (*graphicsDevice)->gdRect.right -
+	(*graphicsDevice)->gdRect.left;
+
+    screen->mwidth      = (screen->width * 254 + 360) / 720;
+    screen->mheight     = (screen->height * 254 + 360) / 720;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -94,8 +134,8 @@ TkpOpenDisplay(
 {
     Display *display;
     Screen *screen;
-    GDHandle graphicsDevice;
     int      fd = 0;
+
     if (gMacDisplay != NULL) {
 	if (strcmp(gMacDisplay->display->display_name, display_name) == 0) {
 	    return gMacDisplay;
@@ -104,54 +144,57 @@ TkpOpenDisplay(
 	}
     }
     InitCursor();
-     
-    graphicsDevice = GetMainDevice();
+
     display = (Display *) ckalloc(sizeof(Display));
+    screen  = (Screen *) ckalloc(sizeof(Screen));
+    bzero(display, sizeof(Display));
+    bzero(screen, sizeof(Screen));
+
     display->resource_alloc = MacXIdAlloc;
-    screen = (Screen *) ckalloc(sizeof(Screen) * 2);
+    display->request        = 0;
+    display->qlen           = 0;
+    display->fd             = fd;
+    display->screens        = screen;
+    display->nscreens       = 1;
     display->default_screen = 0;
-    display->request = 0;
-    display->nscreens = 1;
-    display->screens = screen;
-    display->display_name = macScreenName;
-    display->qlen = 0;
-    display->fd = fd;
-    display->proto_major_version = 10;
+    display->display_name   = macScreenName;
+
     Gestalt(gestaltQuickdrawVersion, (long*)&display->proto_minor_version);
+    display->proto_major_version = 10;
     display->proto_minor_version -= gestaltMacOSXQD;
     display->vendor = "Apple";
     Gestalt(gestaltSystemVersion, (long*)&display->release);
-    
-    screen->root = ROOT_ID;
-    screen->display = display;
-    screen->root_depth = (*(*graphicsDevice)->gdPMap)->cmpSize *
-                               (*(*graphicsDevice)->gdPMap)->cmpCount;	
-    screen->height = (*graphicsDevice)->gdRect.bottom -
-	(*graphicsDevice)->gdRect.top;
-    screen->width = (*graphicsDevice)->gdRect.right -
-	(*graphicsDevice)->gdRect.left;
-    
-    screen->mwidth = (screen->width * 254 + 360) / 720;
-    screen->mheight = (screen->height * 254 + 360) / 720;
+
+    /*
+     * These screen bits never change
+     */
+    screen->root        = ROOT_ID;
+    screen->display     = display;
     screen->black_pixel = 0x00000000;
     screen->white_pixel = 0x00FFFFFF;
+
     screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
-    screen->root_visual->visualid = 0;
-    screen->root_visual->class = TrueColor;
-    screen->root_visual->red_mask = 0x00FF0000;
-    screen->root_visual->green_mask = 0x0000FF00;
-    screen->root_visual->blue_mask = 0x000000FF;
+    screen->root_visual->visualid     = 0;
+    screen->root_visual->class        = TrueColor;
+    screen->root_visual->red_mask     = 0x00FF0000;
+    screen->root_visual->green_mask   = 0x0000FF00;
+    screen->root_visual->blue_mask    = 0x000000FF;
     screen->root_visual->bits_per_rgb = 24;
-    screen->root_visual->map_entries = 256;
+    screen->root_visual->map_entries  = 256;
+
+    /*
+     * Initialize screen bits that may change
+     */
+    TkMacOSXDisplayChanged(display);
 
     gMacDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
-    
+
     /*
      * This is the quickest way to make sure that all the *Init
      * flags get properly initialized
      */
-    
-    bzero (gMacDisplay, sizeof (TkDisplay));
+
+    bzero(gMacDisplay, sizeof(TkDisplay));
     gMacDisplay->display = display;
     return gMacDisplay;
 }
@@ -178,7 +221,7 @@ TkpCloseDisplay(
 {
     Display *display = displayPtr->display;
     if (gMacDisplay != displayPtr) {
-        panic("TkpCloseDisplay: tried to call TkpCloseDisplay on bad display");
+        Tcl_Panic("TkpCloseDisplay: tried to call TkpCloseDisplay on bad display");
     }
 
     gMacDisplay = NULL;
@@ -319,7 +362,7 @@ DefaultErrorHandler(
      * This call should never be called.  Tk replaces
      * it with its own error handler.
      */
-    panic("Warning hit bogus error handler!");
+    Tcl_Panic("Warning hit bogus error handler!");
     return 0;
 }
 
@@ -365,7 +408,7 @@ XGetImage(display, d, x, y, width, height, plane_mask, format)
     int format;
 { 
     XImage *   imagePtr = NULL;
-    Pixmap     pixmap = NULL;
+    Pixmap     pixmap = (Pixmap) NULL;
     Tk_Window  win = (Tk_Window) ((MacDrawable *) d)->winPtr;
     GC         gc;
     int        depth = 32;
@@ -475,6 +518,7 @@ XBell(
     SysBeep(percent);
 }
 
+#if 0
 void
 XSetWMNormalHints(
     Display* display,
@@ -496,6 +540,7 @@ XAllocSizeHints()
     
     return NULL;
 }
+#endif
 
 XImage * 
 XCreateImage(
@@ -707,6 +752,14 @@ Tk_FreeXId (
 {
     /* no-op function needed for stubs implementation. */
 }
+
+int
+XSync (Display *display, Bool flag)
+{
+    TkMacOSXFlushWindows();
+    display->request++;
+    return 0;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -746,7 +799,7 @@ TkGetServerInfo(
  * Image stuff 
  */
 
-int 
+static int 
 TkMacOSXXDestroyImage(
     XImage *image)
 {
@@ -755,16 +808,17 @@ TkMacOSXXDestroyImage(
     return 0;
 }
 
-unsigned long 
+static unsigned long 
 TkMacOSXXGetPixel(
     XImage *image,
     int x,
     int y)
 {
-    CGrafPtr grafPtr;
+    CGrafPtr grafPtr, oldPort;
     RGBColor cPix;
     unsigned long r, g, b, c;
     grafPtr = (CGrafPtr)image->data;
+    GetPort(&oldPort);
     SetPort(grafPtr);
     GetCPixel(x,y,&cPix);
     if (image->obdata) {
@@ -778,20 +832,22 @@ TkMacOSXXGetPixel(
         b = cPix . blue;
     }
     c = (r<<16)|(g<<8)|(b);
+    SetPort(oldPort);
     return c;
 }
 
-int 
+static int 
 TkMacOSXXPutPixel(
     XImage *image,
     int x,
     int y,
     unsigned long pixel)
 {
-    CGrafPtr grafPtr;
+    CGrafPtr grafPtr, oldPort;
     RGBColor cPix;
     unsigned long r, g, b;
     grafPtr = (CGrafPtr)image->data;
+    GetPort(&oldPort);
     SetPort(grafPtr);
     r  = (pixel & image->red_mask)>>16;
     g  = (pixel & image->green_mask)>>8;
@@ -807,10 +863,11 @@ TkMacOSXXPutPixel(
         cPix . blue = b;
     }
     SetCPixel(x,y,&cPix);
+    SetPort(oldPort);
     return 0;
 }
 
-XImage *
+static XImage *
 TkMacOSXXSubImage(
     XImage *image,
     int x,
@@ -822,7 +879,7 @@ TkMacOSXXSubImage(
     return NULL;
 }
 
-int 
+static int 
 TkMacOSXXAddPixel(
     XImage *image,
     long value)
@@ -917,7 +974,7 @@ XStringListToTextProperty(
     Debugger();
     return (Status) 0;
 }
-void		
+void
 XSetWMClientMachine(
     Display* display, 
     Window w, 
@@ -925,7 +982,7 @@ XSetWMClientMachine(
 {
     Debugger();
 }
-XIC		
+XIC
 XCreateIC(
     void)
 {
