@@ -5,12 +5,13 @@
  *	Windows-specific parts of Tk, but aren't used by the rest of
  *	Tk.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-2000 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinInt.h,v 1.3 1999/02/04 21:01:35 stanton Exp $
+ * RCS: @(#) $Id: tkWinInt.h,v 1.14.2.5 2004/09/23 01:49:08 hobbs Exp $
  */
 
 #ifndef _TKWININT
@@ -28,6 +29,11 @@
 #include "tkWin.h"
 #endif
 
+#ifndef _TKPORT
+#include "tkPort.h"
+#endif
+
+
 /*
  * Define constants missing from older Win32 SDK header files.
  */
@@ -36,8 +42,6 @@
 #define WS_EX_TOOLWINDOW	0x00000080L 
 #endif
 
-typedef struct TkFontAttributes TkFontAttributes;
-
 /*
  * The TkWinDCState is used to save the state of a device context
  * so that it can be restored later.
@@ -45,6 +49,7 @@ typedef struct TkFontAttributes TkFontAttributes;
 
 typedef struct TkWinDCState {
     HPALETTE palette;
+    int bkmode;
 } TkWinDCState;
 
 /*
@@ -86,11 +91,11 @@ typedef union {
  * The following macros are used to retrieve internal values from a Drawable.
  */
 
-#define TkWinGetHWND(w) (((TkWinDrawable *) w)->window.handle)
-#define TkWinGetWinPtr(w) (((TkWinDrawable*)w)->window.winPtr)
-#define TkWinGetHBITMAP(w) (((TkWinDrawable*)w)->bitmap.handle)
-#define TkWinGetColormap(w) (((TkWinDrawable*)w)->bitmap.colormap)
-#define TkWinGetHDC(w) (((TkWinDrawable *) w)->winDC.hdc)
+#define TkWinGetHWND(w)		(((TkWinDrawable *) w)->window.handle)
+#define TkWinGetWinPtr(w)	(((TkWinDrawable *) w)->window.winPtr)
+#define TkWinGetHBITMAP(w)	(((TkWinDrawable *) w)->bitmap.handle)
+#define TkWinGetColormap(w)	(((TkWinDrawable *) w)->bitmap.colormap)
+#define TkWinGetHDC(w)		(((TkWinDrawable *) w)->winDC.hdc)
 
 /*
  * The following structure is used to encapsulate palette information.
@@ -118,21 +123,16 @@ typedef struct {
  */
 
 #define TK_WIN_TOPLEVEL_CLASS_NAME "TkTopLevel"
+#define TK_WIN_TOPLEVEL_NOCDC_CLASS_NAME "TkTopLevelNoCDC"
 #define TK_WIN_CHILD_CLASS_NAME "TkChild"
 
-/*
- * The following variable indicates whether we are restricted to Win32s
- * GDI calls.
- */
-
-extern int tkpIsWin32s;
-
 /*
  * The following variable is a translation table between X gc functions and
- * Win32 raster op modes.
+ * Win32 raster and BitBlt op modes.
  */
 
 extern int tkpWinRopModes[];
+extern int tkpWinBltModes[];
 
 /*
  * The following defines are used with TkWinGetBorderPixels to get the
@@ -146,50 +146,80 @@ extern int tkpWinRopModes[];
  * Internal procedures used by more than one source file.
  */
 
-extern void		TkWinCancelMouseTimer _ANSI_ARGS_((void));
-extern LRESULT CALLBACK	TkWinChildProc _ANSI_ARGS_((HWND hwnd, UINT message,
+#include "tkIntPlatDecls.h"
+
+/*
+ * We need to specially add the TkWinChildProc because of the special
+ * prototype it has (doesn't fit into stubs schema)
+ */
+#ifdef BUILD_tk
+#undef TCL_STORAGE_CLASS
+#define TCL_STORAGE_CLASS DLLEXPORT
+#endif
+
+EXTERN LRESULT CALLBACK	TkWinChildProc _ANSI_ARGS_((HWND hwnd, UINT message,
 			    WPARAM wParam, LPARAM lParam));
-extern void		TkWinClipboardRender _ANSI_ARGS_((TkDisplay *dispPtr,
-			    UINT format));
-extern LRESULT		TkWinEmbeddedEventProc _ANSI_ARGS_((HWND hwnd,
-			    UINT message, WPARAM wParam, LPARAM lParam));
-extern void		TkWinFillRect _ANSI_ARGS_((HDC dc, int x, int y,
-			    int width, int height, int pixel));
-extern COLORREF		TkWinGetBorderPixels _ANSI_ARGS_((Tk_Window tkwin,
-			    Tk_3DBorder border, int which));
-extern HDC		TkWinGetDrawableDC _ANSI_ARGS_((Display *display,
-			    Drawable d, TkWinDCState* state));
-extern int		TkWinGetModifierState _ANSI_ARGS_((void));
-extern HPALETTE		TkWinGetSystemPalette _ANSI_ARGS_((void));
-extern HWND		TkWinGetWrapperWindow _ANSI_ARGS_((Tk_Window tkwin));
-extern int		TkWinHandleMenuEvent _ANSI_ARGS_((HWND *phwnd,
-			    UINT *pMessage, WPARAM *pwParam, LPARAM *plParam,
-			    LRESULT *plResult));
-extern int		TkWinIndexOfColor _ANSI_ARGS_((XColor *colorPtr));
-extern void		TkWinPointerDeadWindow _ANSI_ARGS_((TkWindow *winPtr));
-extern void		TkWinPointerEvent _ANSI_ARGS_((HWND hwnd, int x,
-			    int y));
-extern void		TkWinPointerInit _ANSI_ARGS_((void));
-extern LRESULT 		TkWinReflectMessage _ANSI_ARGS_((HWND hwnd,
-			    UINT message, WPARAM wParam, LPARAM lParam));
-extern void		TkWinReleaseDrawableDC _ANSI_ARGS_((Drawable d,
-			    HDC hdc, TkWinDCState* state));
-extern LRESULT		TkWinResendEvent _ANSI_ARGS_((WNDPROC wndproc,
-			    HWND hwnd, XEvent *eventPtr));
-extern HPALETTE		TkWinSelectPalette _ANSI_ARGS_((HDC dc,
-			    Colormap colormap));
-extern void		TkWinSetMenu _ANSI_ARGS_((Tk_Window tkwin,
-			    HMENU hMenu));
-extern void		TkWinSetWindowPos _ANSI_ARGS_((HWND hwnd,
-			    HWND siblingHwnd, int pos));
-extern void		TkWinUpdateCursor _ANSI_ARGS_((TkWindow *winPtr));
-extern void		TkWinWmCleanup _ANSI_ARGS_((HINSTANCE hInstance));
-extern HWND		TkWinWmFindEmbedAssociation _ANSI_ARGS_((
-			    TkWindow *winPtr));
-extern void		TkWinWmStoreEmbedAssociation _ANSI_ARGS_((
-			    TkWindow *winPtr, HWND hwnd));
-extern void		TkWinXCleanup _ANSI_ARGS_((HINSTANCE hInstance));
-extern void 		TkWinXInit _ANSI_ARGS_((HINSTANCE hInstance));
+
+/*
+ * Special proc needed as tsd accessor function between
+ * tkWinX.c:GenerateXEvent and tkWinClipboard.c:UpdateClipboard
+ */
+EXTERN void	TkWinUpdatingClipboard(int mode);
+
+/*
+ * Used by tkWinDialog.c to associate the right icon with tk_messageBox
+ */
+EXTERN HICON	TkWinGetIcon(Tk_Window tkw, DWORD iconsize);
+
+/*
+ * Used by tkWinX.c on for certain system display change messages
+ */
+EXTERN void	TkWinDisplayChanged(Display *display);
+
+/*
+ * The following structure keeps track of whether we are using the 
+ * multi-byte or the wide-character interfaces to the operating system.
+ * System calls should be made through the following function table.
+ *
+ * While some system calls need to use this A/W jump-table, it is not
+ * necessary for all calls to do it, which is why you won't see this
+ * used throughout the Tk code, but only in key areas. -- hobbs
+ */
+
+typedef struct TkWinProcs {
+    int useWide;
+    LRESULT (WINAPI *callWindowProc)(WNDPROC lpPrevWndFunc, HWND hWnd,
+	    UINT Msg, WPARAM wParam, LPARAM lParam);
+    LRESULT (WINAPI *defWindowProc)(HWND hWnd, UINT Msg, WPARAM wParam,
+	    LPARAM lParam);
+    ATOM (WINAPI *registerClass)(CONST WNDCLASS *lpWndClass);
+    BOOL (WINAPI *setWindowText)(HWND hWnd, LPCTSTR lpString);
+    HWND (WINAPI *createWindowEx)(DWORD dwExStyle, LPCTSTR lpClassName,
+	    LPCTSTR lpWindowName, DWORD dwStyle, int x, int y,
+	    int nWidth, int nHeight, HWND hWndParent, HMENU hMenu,
+	    HINSTANCE hInstance, LPVOID lpParam);
+    BOOL (WINAPI *insertMenu)(HMENU hMenu, UINT uPosition, UINT uFlags,
+	    UINT uIDNewItem, LPCTSTR lpNewItem);
+} TkWinProcs;
+
+EXTERN TkWinProcs *tkWinProcs;
+
+#undef TCL_STORAGE_CLASS
+#define TCL_STORAGE_CLASS DLLIMPORT
+
+/*
+ * The following allows us to cache these encoding for multiple functions.
+ */
+
+
+extern Tcl_Encoding TkWinGetKeyInputEncoding _ANSI_ARGS_((void));
+extern Tcl_Encoding TkWinGetUnicodeEncoding _ANSI_ARGS_((void));
+
+/*
+ * Values returned by TkWinGetPlatformTheme.
+ */
+#define TK_THEME_WIN_CLASSIC    1
+#define TK_THEME_WIN_XP         2
 
 #endif /* _TKWININT */
 
