@@ -10,30 +10,86 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinX.c,v 1.10.2.2 2001/04/06 00:13:24 hobbs Exp $
+ * RCS: @(#) $Id: tkWinX.c,v 1.25.2.6 2005/02/28 22:10:26 hobbs Exp $
  */
 
 #include "tkWinInt.h"
 
+/*
+ * The w32api 1.1 package (included in Mingw 1.1) does not define _WIN32_IE
+ * by default. Define it here to gain access to the InitCommonControlsEx API
+ * in commctrl.h.
+ */
+
+#ifndef _WIN32_IE
+#define _WIN32_IE 0x0300
+#endif
+
+#include <commctrl.h>
+
 /*
  * The zmouse.h file includes the definition for WM_MOUSEWHEEL.
  */
 
 #include <zmouse.h>
 
+/*
+ * imm.h is needed by HandleIMEComposition
+ */
+
+#include <imm.h>
+
+static TkWinProcs asciiProcs = {
+    0,
+
+    (LRESULT (WINAPI *)(WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg,
+	    WPARAM wParam, LPARAM lParam)) CallWindowProcA,
+    (LRESULT (WINAPI *)(HWND hWnd, UINT Msg, WPARAM wParam,
+	    LPARAM lParam)) DefWindowProcA,
+    (ATOM (WINAPI *)(CONST WNDCLASS *lpWndClass)) RegisterClassA,
+    (BOOL (WINAPI *)(HWND hWnd, LPCTSTR lpString)) SetWindowTextA,
+    (HWND (WINAPI *)(DWORD dwExStyle, LPCTSTR lpClassName,
+	    LPCTSTR lpWindowName, DWORD dwStyle, int x, int y,
+	    int nWidth, int nHeight, HWND hWndParent, HMENU hMenu,
+	    HINSTANCE hInstance, LPVOID lpParam)) CreateWindowExA,
+    (BOOL (WINAPI *)(HMENU hMenu, UINT uPosition, UINT uFlags,
+	    UINT uIDNewItem, LPCTSTR lpNewItem)) InsertMenuA,
+};
+
+static TkWinProcs unicodeProcs = {
+    1,
+
+    (LRESULT (WINAPI *)(WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg,
+	    WPARAM wParam, LPARAM lParam)) CallWindowProcW,
+    (LRESULT (WINAPI *)(HWND hWnd, UINT Msg, WPARAM wParam,
+	    LPARAM lParam)) DefWindowProcW,
+    (ATOM (WINAPI *)(CONST WNDCLASS *lpWndClass)) RegisterClassW,
+    (BOOL (WINAPI *)(HWND hWnd, LPCTSTR lpString)) SetWindowTextW,
+    (HWND (WINAPI *)(DWORD dwExStyle, LPCTSTR lpClassName,
+	    LPCTSTR lpWindowName, DWORD dwStyle, int x, int y,
+	    int nWidth, int nHeight, HWND hWndParent, HMENU hMenu,
+	    HINSTANCE hInstance, LPVOID lpParam)) CreateWindowExW,
+    (BOOL (WINAPI *)(HMENU hMenu, UINT uPosition, UINT uFlags,
+	    UINT uIDNewItem, LPCTSTR lpNewItem)) InsertMenuW,
+};
+
+TkWinProcs *tkWinProcs;
+
 /*
  * Declarations of static variables used in this file.
  */
 
 static char winScreenName[] = ":0"; /* Default name of windows display. */
-static HINSTANCE tkInstance;        /* Application instance handle. */
+static HINSTANCE tkInstance = NULL; /* Application instance handle. */
 static int childClassInitialized;   /* Registered child class? */
 static WNDCLASS childClass;	    /* Window class for child windows. */
-static int tkPlatformId;	    /* version of Windows platform */
+static int tkPlatformId = 0;	    /* version of Windows platform */
+static int tkWinTheme = 0;          /* See TkWinGetPlatformTheme */
 static Tcl_Encoding keyInputEncoding = NULL;/* The current character
 				     * encoding for keyboard input */
 static int keyInputCharset = -1;    /* The Win32 CHARSET for the keyboard
 				     * encoding */
+static Tcl_Encoding unicodeEncoding = NULL; /* unicode encoding */
 
 /*
  * Thread local storage.  Notice that now each thread must have its
@@ -90,8 +146,14 @@ TkGetServerInfo(interp, tkwin)
 
     os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
     GetVersionEx(&os);
-    sprintf(buffer, "Windows %d.%d %d Win32", os.dwMajorVersion,
-	    os.dwMinorVersion, os.dwBuildNumber);
+    sprintf(buffer, "Windows %d.%d %d %s", os.dwMajorVersion,
+	    os.dwMinorVersion, os.dwBuildNumber,
+#ifdef _WIN64
+	    "Win64"
+#else
+	    "Win32"
+#endif
+	);
     Tcl_SetResult(interp, buffer, TCL_VOLATILE);
 }
 
@@ -114,9 +176,36 @@ TkGetServerInfo(interp, tkwin)
 HINSTANCE
 Tk_GetHINSTANCE()
 {
+    if (tkInstance == NULL) {
+	tkInstance = GetModuleHandle(NULL);
+    }
     return tkInstance;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinSetHINSTANCE --
+ *
+ *	Sets the global instance handle used by the Tk library.
+ *	This should be called by DllMain.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkWinSetHINSTANCE(hInstance)
+    HINSTANCE hInstance;
+{
+    tkInstance = hInstance;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -137,18 +226,25 @@ void
 TkWinXInit(hInstance)
     HINSTANCE hInstance;
 {
-    OSVERSIONINFO os;
-
     if (childClassInitialized != 0) {
 	return;
     }
     childClassInitialized = 1;
 
-    tkInstance = hInstance;
+    if (TkWinGetPlatformId() == VER_PLATFORM_WIN32_NT) {
+	/*
+	 * This is necessary to enable the use of themeable elements on XP,
+	 * so we don't even try and call it for Win9*.
+	 */
 
-    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&os);
-    tkPlatformId = os.dwPlatformId;
+	INITCOMMONCONTROLSEX comctl;
+	ZeroMemory(&comctl, sizeof(comctl));
+	(void) InitCommonControlsEx(&comctl);
+
+	tkWinProcs = &unicodeProcs;
+    } else {
+	tkWinProcs = &asciiProcs;
+    }
 
     /*
      * When threads are enabled, we cannot use CLASSDC because
@@ -183,6 +279,11 @@ TkWinXInit(hInstance)
     if (!RegisterClass(&childClass)) {
 	panic("Unable to register TkChild class");
     }
+
+    /*
+     * Make sure we cleanup on finalize.
+     */
+    TkCreateExitHandler(TkWinXCleanup, (ClientData) hInstance);
 }
 
 /*
@@ -202,9 +303,10 @@ TkWinXInit(hInstance)
  */
 
 void
-TkWinXCleanup(hInstance)
-    HINSTANCE hInstance;
+TkWinXCleanup(clientData)
+    ClientData clientData;
 {
+    HINSTANCE hInstance = (HINSTANCE) clientData;
     /*
      * Clean up our own class.
      */
@@ -214,6 +316,11 @@ TkWinXCleanup(hInstance)
         UnregisterClass(TK_WIN_CHILD_CLASS_NAME, hInstance);
     }
 
+    if (unicodeEncoding != NULL) {
+	Tcl_FreeEncoding(unicodeEncoding);
+	unicodeEncoding = NULL;
+    }
+
     /*
      * And let the window manager clean up its own class(es).
      */
@@ -227,7 +334,7 @@ TkWinXCleanup(hInstance)
  * TkWinGetPlatformId --
  *
  *	Determines whether running under NT, 95, or Win32s, to allow 
- *	runtime conditional code.
+ *	runtime conditional code.  Win32s is no longer supported.
  *
  * Results:
  *	The return value is one of:
@@ -241,12 +348,73 @@ TkWinXCleanup(hInstance)
  *----------------------------------------------------------------------
  */
 
-int		
+int
 TkWinGetPlatformId()
 {
+    if (tkPlatformId == 0) {
+	OSVERSIONINFO os;
+
+	os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+	GetVersionEx(&os);
+	tkPlatformId = os.dwPlatformId;
+        
+        /* Set tkWinTheme to be TK_THEME_WIN_XP or TK_THEME_WIN_CLASSIC.
+         * The TK_THEME_WIN_CLASSIC could be set even when running
+         * under XP if the windows classic theme was selected. */
+	if ((os.dwPlatformId == VER_PLATFORM_WIN32_NT) &&
+	        (os.dwMajorVersion == 5 && os.dwMinorVersion == 1)) {
+	    HKEY   hKey;
+	    LPCSTR szSubKey  = TEXT("Control Panel\\Appearance");
+	    LPCSTR szCurrent = TEXT("Current");
+	    DWORD  dwSize = 200;
+	    char pBuffer[200];
+	    memset(pBuffer, 0, dwSize);
+	    if (RegOpenKeyEx(HKEY_CURRENT_USER, szSubKey, 0L,
+                    KEY_READ, &hKey) != ERROR_SUCCESS) {
+                tkWinTheme = TK_THEME_WIN_XP;
+	    } else {
+	        RegQueryValueEx(hKey, szCurrent, NULL, NULL, pBuffer, &dwSize);
+	        RegCloseKey(hKey);
+	        if (strcmp(pBuffer, "Windows Standard") == 0) {
+	            tkWinTheme = TK_THEME_WIN_CLASSIC;
+	        } else {
+	            tkWinTheme = TK_THEME_WIN_XP;
+	        }
+	    }
+	} else {
+	    tkWinTheme = TK_THEME_WIN_CLASSIC;
+	}
+    }
     return tkPlatformId;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinGetPlatformTheme --
+ *
+ *	Return the Windows drawing style we should be using.
+ *
+ * Results:
+ *	The return value is one of:
+ *	    TK_THEME_WIN_CLASSIC		95/98/NT or XP in classic mode
+ *	    TK_THEME_WIN_XP	                XP not in classic mode
+ *
+ * Side effects:
+ *	Could invoke TkWinGetPlatformId.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkWinGetPlatformTheme()
+{
+    if (tkPlatformId == 0) {
+        TkWinGetPlatformId();
+    }
+    return tkWinTheme;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -264,10 +432,10 @@ TkWinGetPlatformId()
  *----------------------------------------------------------------------
  */
 
-char *
+CONST char *
 TkGetDefaultScreenName(interp, screenName)
     Tcl_Interp *interp;		/* Not used. */
-    char *screenName;		/* If NULL, use default string. */
+    CONST char *screenName;	/* If NULL, use default string. */
 {
     if ((screenName == NULL) || (screenName[0] == '\0')) {
 	screenName = winScreenName;
@@ -275,6 +443,100 @@ TkGetDefaultScreenName(interp, screenName)
     return screenName;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinDisplayChanged --
+ *
+ *	Called to set up initial screen info or when an event indicated
+ *	display (screen) change.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	May change info regarding the screen.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkWinDisplayChanged(Display *display)
+{
+    HDC dc;
+    Screen *screen;
+
+    if (display == NULL || display->screens == NULL) {
+	return;
+    }
+    screen = display->screens;
+
+    dc = GetDC(NULL);
+    screen->width = GetDeviceCaps(dc, HORZRES);
+    screen->height = GetDeviceCaps(dc, VERTRES);
+    screen->mwidth = MulDiv(screen->width, 254,
+	    GetDeviceCaps(dc, LOGPIXELSX) * 10);
+    screen->mheight = MulDiv(screen->height, 254,
+	    GetDeviceCaps(dc, LOGPIXELSY) * 10);
+
+    /*
+     * On windows, when creating a color bitmap, need two pieces of
+     * information: the number of color planes and the number of pixels per
+     * plane.  Need to remember both quantities so that when constructing an
+     * HBITMAP for offscreen rendering, we can specify the correct value for
+     * the number of planes.  Otherwise the HBITMAP won't be compatible with
+     * the HWND and we'll just get blank spots copied onto the screen.
+     */
+
+    screen->ext_data = (XExtData *) GetDeviceCaps(dc, PLANES);
+    screen->root_depth = GetDeviceCaps(dc, BITSPIXEL) * (int) screen->ext_data;
+
+    if (screen->root_visual != NULL) {
+	ckfree((char *) screen->root_visual);
+    }
+    screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
+    screen->root_visual->visualid = 0;
+    if (GetDeviceCaps(dc, RASTERCAPS) & RC_PALETTE) {
+	screen->root_visual->map_entries = GetDeviceCaps(dc, SIZEPALETTE);
+	screen->root_visual->class = PseudoColor;
+	screen->root_visual->red_mask = 0x0;
+	screen->root_visual->green_mask = 0x0;
+	screen->root_visual->blue_mask = 0x0;
+    } else if (screen->root_depth == 4) {
+	screen->root_visual->class = StaticColor;
+	screen->root_visual->map_entries = 16;
+    } else if (screen->root_depth == 8) {
+	screen->root_visual->class = StaticColor;
+	screen->root_visual->map_entries = 256;
+    } else if (screen->root_depth == 12) {
+	screen->root_visual->class = TrueColor;
+	screen->root_visual->map_entries = 32;
+	screen->root_visual->red_mask = 0xf0;
+	screen->root_visual->green_mask = 0xf000;
+	screen->root_visual->blue_mask = 0xf00000;
+    } else if (screen->root_depth == 16) {
+	screen->root_visual->class = TrueColor;
+	screen->root_visual->map_entries = 64;
+	screen->root_visual->red_mask = 0xf8;
+	screen->root_visual->green_mask = 0xfc00;
+	screen->root_visual->blue_mask = 0xf80000;
+    } else if (screen->root_depth >= 24) {
+	screen->root_visual->class = TrueColor;
+	screen->root_visual->map_entries = 256;
+	screen->root_visual->red_mask = 0xff;
+	screen->root_visual->green_mask = 0xff00;
+	screen->root_visual->blue_mask = 0xff0000;
+    }
+    screen->root_visual->bits_per_rgb = screen->root_depth;
+    ReleaseDC(NULL, dc);
+
+    if (screen->cmap != None) {
+	XFreeColormap(display, screen->cmap);
+    }
+    screen->cmap = XCreateColormap(display, None, screen->root_visual,
+	    AllocNone);
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -294,10 +556,9 @@ TkGetDefaultScreenName(interp, screenName)
 
 TkDisplay *
 TkpOpenDisplay(display_name)
-    char *display_name;
+    CONST char *display_name;
 {
     Screen *screen;
-    HDC dc;
     TkWinDrawable *twdPtr;
     Display *display;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
@@ -313,25 +574,20 @@ TkpOpenDisplay(display_name)
     }
 
     display = (Display *) ckalloc(sizeof(Display));
+    ZeroMemory(display, sizeof(Display));
+
     display->display_name = (char *) ckalloc(strlen(display_name)+1);
     strcpy(display->display_name, display_name);
 
     display->cursor_font = 1;
-    display->nscreens = 1;
-    display->request = 1;
-    display->qlen = 0;
+    display->nscreens    = 1;
+    display->request     = 1;
+    display->qlen        = 0;
 
     screen = (Screen *) ckalloc(sizeof(Screen));
+    ZeroMemory(screen, sizeof(Screen));
     screen->display = display;
 
-    dc = GetDC(NULL);
-    screen->width = GetDeviceCaps(dc, HORZRES);
-    screen->height = GetDeviceCaps(dc, VERTRES);
-    screen->mwidth = MulDiv(screen->width, 254,
-	    GetDeviceCaps(dc, LOGPIXELSX) * 10);
-    screen->mheight = MulDiv(screen->height, 254,
-	    GetDeviceCaps(dc, LOGPIXELSY) * 10);
-    
     /*
      * Set up the root window.
      */
@@ -344,57 +600,6 @@ TkpOpenDisplay(display_name)
     twdPtr->window.winPtr = NULL;
     twdPtr->window.handle = NULL;
     screen->root = (Window)twdPtr;
- 
-    /*
-     * On windows, when creating a color bitmap, need two pieces of 
-     * information: the number of color planes and the number of 
-     * pixels per plane.  Need to remember both quantities so that
-     * when constructing an HBITMAP for offscreen rendering, we can
-     * specify the correct value for the number of planes.  Otherwise
-     * the HBITMAP won't be compatible with the HWND and we'll just
-     * get blank spots copied onto the screen.
-     */
-
-    screen->ext_data = (XExtData *) GetDeviceCaps(dc, PLANES);
-    screen->root_depth = GetDeviceCaps(dc, BITSPIXEL) * (int) screen->ext_data;
-
-    screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
-    screen->root_visual->visualid = 0;
-    if (GetDeviceCaps(dc, RASTERCAPS) & RC_PALETTE) {
-	screen->root_visual->map_entries = GetDeviceCaps(dc, SIZEPALETTE);
-	screen->root_visual->class = PseudoColor;
-	screen->root_visual->red_mask = 0x0;
-	screen->root_visual->green_mask = 0x0;
-	screen->root_visual->blue_mask = 0x0;
-    } else {
-	if (screen->root_depth == 4) {
-	    screen->root_visual->class = StaticColor;
-	    screen->root_visual->map_entries = 16;
-	} else if (screen->root_depth == 8) {
-	    screen->root_visual->class = StaticColor;
-	    screen->root_visual->map_entries = 256;
-	} else if (screen->root_depth == 12) {
-	    screen->root_visual->class = TrueColor;
-	    screen->root_visual->map_entries = 32;
-	    screen->root_visual->red_mask = 0xf0;
-	    screen->root_visual->green_mask = 0xf000;
-	    screen->root_visual->blue_mask = 0xf00000;
-	} else if (screen->root_depth == 16) {
-	    screen->root_visual->class = TrueColor;
-	    screen->root_visual->map_entries = 64;
-	    screen->root_visual->red_mask = 0xf8;
-	    screen->root_visual->green_mask = 0xfc00;
-	    screen->root_visual->blue_mask = 0xf80000;
-	} else if (screen->root_depth >= 24) {
-	    screen->root_visual->class = TrueColor;
-	    screen->root_visual->map_entries = 256;
-	    screen->root_visual->red_mask = 0xff;
-	    screen->root_visual->green_mask = 0xff00;
-	    screen->root_visual->blue_mask = 0xff0000;
-	}
-    }
-    screen->root_visual->bits_per_rgb = screen->root_depth;
-    ReleaseDC(NULL, dc);
 
     /*
      * Note that these pixel values are not palette relative.
@@ -402,15 +607,19 @@ TkpOpenDisplay(display_name)
 
     screen->white_pixel = RGB(255, 255, 255);
     screen->black_pixel = RGB(0, 0, 0);
+    screen->cmap        = None;
 
     display->screens		= screen;
     display->nscreens		= 1;
     display->default_screen	= 0;
-    screen->cmap = XCreateColormap(display, None, screen->root_visual,
-	    AllocNone);
+
+    TkWinDisplayChanged(display);
+
     tsdPtr->winDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
+    ZeroMemory(tsdPtr->winDisplay, sizeof(TkDisplay));
     tsdPtr->winDisplay->display = display;
     tsdPtr->updatingClipboard = FALSE;
+
     return tsdPtr->winDisplay;
 }
 
@@ -477,7 +686,54 @@ TkpCloseDisplay(dispPtr)
         ckfree((char *) display->screens);
     }
     ckfree((char *) display);
-    ckfree((char *) dispPtr);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkClipCleanup --
+ *
+ *	This procedure is called to cleanup resources associated with
+ *	claiming clipboard ownership and for receiving selection get
+ *	results.  This function is called in tkWindow.c.  This has to be
+ *	called by the display cleanup function because we still need the
+ *	access display elements.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Resources are freed - the clipboard may no longer be used.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkClipCleanup(dispPtr)
+    TkDisplay *dispPtr;	/* display associated with clipboard */
+{
+    if (dispPtr->clipWindow != NULL) {
+	/*
+	 * Force the clipboard to be rendered if we are the clipboard owner.
+	 */
+
+	HWND hwnd = Tk_GetHWND(Tk_WindowId(dispPtr->clipWindow));
+	if (GetClipboardOwner() == hwnd) {
+	    OpenClipboard(hwnd);
+	    EmptyClipboard();
+	    TkWinClipboardRender(dispPtr, CF_TEXT);
+	    CloseClipboard();
+	}
+
+	Tk_DeleteSelHandler(dispPtr->clipWindow, dispPtr->clipboardAtom,
+		dispPtr->applicationAtom);
+	Tk_DeleteSelHandler(dispPtr->clipWindow, dispPtr->clipboardAtom,
+		dispPtr->windowAtom);
+
+	Tk_DestroyWindow(dispPtr->clipWindow);
+	Tcl_Release((ClientData) dispPtr->clipWindow);
+	dispPtr->clipWindow = NULL;
+    }
 }
 
 /*
@@ -740,7 +996,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	case WM_SETFOCUS:
 	case WM_KILLFOCUS: {
 	    TkWindow *otherWinPtr = (TkWindow *)Tk_HWNDToWindow((HWND) wParam);
-	    
+
 	    /*
 	     * Compare toplevel windows to avoid reporting focus
 	     * changes within the same toplevel.
@@ -755,6 +1011,15 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	    while (otherWinPtr && !(otherWinPtr->flags & TK_TOP_LEVEL)) {
 		otherWinPtr = otherWinPtr->parentPtr;
 	    }
+
+	    /*
+	     * Do a catch-all Tk_SetCaretPos here to make sure that the
+	     * window receiving focus sets the caret at least once.
+	     */
+	    if (message == WM_SETFOCUS) {
+		Tk_SetCaretPos((Tk_Window) winPtr, 0, 0, 0);
+	    }
+
 	    if (otherWinPtr == winPtr) {
 		return;
 	    }
@@ -763,6 +1028,14 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	    event.type = (message == WM_SETFOCUS) ? FocusIn : FocusOut;
 	    event.xfocus.mode = NotifyNormal;
 	    event.xfocus.detail = NotifyNonlinear;
+
+	    /*
+	     * Destroy the caret if we own it.  If we are moving to another Tk
+	     * window, it will reclaim and reposition it with Tk_SetCaretPos.
+	     */
+	    if (message == WM_KILLFOCUS) {
+		DestroyCaret();
+	    }
 	    break;
 	}
 
@@ -779,14 +1052,14 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		Tk_InternAtom((Tk_Window)winPtr, "CLIPBOARD");
 	    event.xselectionclear.time = TkpGetMS();
 	    break;
-	    
+
 	case WM_MOUSEWHEEL:
 	    /*
 	     * The mouse wheel event is closer to a key event than a
 	     * mouse event in that the message is sent to the window
 	     * that has focus.
 	     */
-	    
+
 	case WM_CHAR:
 	case WM_SYSKEYDOWN:
 	case WM_SYSKEYUP:
@@ -801,7 +1074,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	    /*
 	     * Compute the screen and window coordinates of the event.
 	     */
-	    
+
 	    msgPos = GetMessagePos();
 	    rootPoint = MAKEPOINTS(msgPos);
 	    clientPoint.x = rootPoint.x;
@@ -833,13 +1106,15 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		     * We have invented a new X event type to handle
 		     * this event.  It still uses the KeyPress struct.
 		     * However, the keycode field has been overloaded
-		     * to hold the zDelta of the wheel.
+		     * to hold the zDelta of the wheel.  Set nbytes to 0
+		     * to prevent conversion of the keycode to a keysym
+		     * in TkpGetString. [Bug 1118340].
 		     */
-		    
+
 		    event.type = MouseWheelEvent;
 		    event.xany.send_event = -1;
-		    event.xkey.keycode = (short) HIWORD(wParam);
 		    event.xkey.nbytes = 0;
+		    event.xkey.keycode = (short) HIWORD(wParam);
 		    break;
 		case WM_SYSKEYDOWN:
 		case WM_KEYDOWN:
@@ -850,7 +1125,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		     * event was generated by windows and that the Windows
 		     * extension xkey.trans_chars is filled with the
 		     * MBCS characters that came from the TranslateMessage
-		     * call. 
+		     * call.
 		     */
 
 		    event.type = KeyPress;
@@ -1158,6 +1433,31 @@ TkWinGetKeyInputEncoding()
     return keyInputEncoding;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinGetUnicodeEncoding --
+ *
+ *	Returns the cached unicode encoding.
+ *
+ * Results:
+ *	The unicode encoding.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tcl_Encoding
+TkWinGetUnicodeEncoding()
+{
+    if (unicodeEncoding == NULL) {
+	unicodeEncoding = Tcl_GetEncoding(NULL, "unicode");
+    }
+    return unicodeEncoding;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1198,14 +1498,8 @@ HandleIMEComposition(hwnd, lParam)
     XEvent event;
     char * buff;
     TkWindow *winPtr;
-
-    if (tkPlatformId != VER_PLATFORM_WIN32_NT) {
-        /*
-         * The ImmGetCompositionStringW function works only on WinNT.
-         */
-
-        return 0;
-    }
+    Tcl_Encoding unicodeEncoding = TkWinGetUnicodeEncoding();
+    BOOL isWinNT = (TkWinGetPlatformId() == VER_PLATFORM_WIN32_NT);
 
     if ((lParam & GCS_RESULTSTR) == 0) {
         /*
@@ -1217,11 +1511,39 @@ HandleIMEComposition(hwnd, lParam)
 
     hIMC = ImmGetContext(hwnd);
     if (hIMC) {
-        n = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, NULL, 0);
+	if (isWinNT) {
+	    n = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, NULL, 0);
+	} else {
+	    n = ImmGetCompositionStringA(hIMC, GCS_RESULTSTR, NULL, 0);
+	}
 
-        if (n > 0) {
-            buff = (char*)ckalloc(n);
-            n = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, buff, n);
+        if ((n > 0) && ((buff = (char *) ckalloc(n)) != NULL)) {
+	    if (isWinNT) {
+		n = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, buff, n);
+	    } else {
+		Tcl_DString utfString, unicodeString;
+
+		n = ImmGetCompositionStringA(hIMC, GCS_RESULTSTR, buff, n);
+		Tcl_DStringInit(&utfString);
+		Tcl_ExternalToUtfDString(keyInputEncoding, buff, n,
+			&utfString);
+		Tcl_UtfToExternalDString(unicodeEncoding,
+			Tcl_DStringValue(&utfString), -1, &unicodeString);
+		i = Tcl_DStringLength(&unicodeString);
+		if (n < i) {
+		    /*
+		     * Only alloc more space if we need, otherwise just
+		     * use what we've created.  Don't realloc as that may
+		     * copy data we no longer need.
+		     */
+		    ckfree((char *) buff);
+		    buff = (char *) ckalloc(i);
+		}
+		n = i;
+		memcpy(buff, Tcl_DStringValue(&unicodeString), n);
+		Tcl_DStringFree(&utfString);
+		Tcl_DStringFree(&unicodeString);
+	    }
 
 	    /*
 	     * Set up the fields pertinent to key event.
@@ -1276,7 +1598,7 @@ HandleIMEComposition(hwnd, lParam)
  *
  * Tk_FreeXId --
  *
- *	This inteface is not needed under Windows.
+ *	This interface is not needed under Windows.
  *
  * Results:
  *	None.
@@ -1407,3 +1729,100 @@ TkWinUpdatingClipboard(int mode)
 
     tsdPtr->updatingClipboard = mode;
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_SetCaretPos --
+ *
+ *	This enables correct movement of focus in the MS Magnifier, as well
+ *	as allowing us to correctly position the IME Window.  The following
+ *	Win32 APIs are used to work with MS caret:
+ *
+ *	CreateCaret	DestroyCaret	SetCaretPos	GetCaretPos
+ *
+ *	Only one instance of caret can be active at any time
+ *	(e.g. DestroyCaret API does not take any argument such as handle).
+ *	Since do-it-right approach requires to track the create/destroy
+ *	caret status all the time in a global scope among windows (or
+ *	widgets), we just implement this minimal setup to get the job done.
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	Sets the global Windows caret position.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+Tk_SetCaretPos(Tk_Window tkwin, int x, int y, int height)
+{
+    static HWND caretHWND = NULL;
+    TkCaret *caretPtr = &(((TkWindow *) tkwin)->dispPtr->caret);
+    Window win;
+
+    /*
+     * Prevent processing anything if the values haven't changed.
+     * Windows only has one display, so we can do this with statics.
+     */
+    if ((caretPtr->winPtr == ((TkWindow *) tkwin))
+	    && (caretPtr->x == x) && (caretPtr->y == y)) {
+	return;
+    }
+
+    caretPtr->winPtr = ((TkWindow *) tkwin);
+    caretPtr->x = x;
+    caretPtr->y = y;
+    caretPtr->height = height;
+
+    /*
+     * We adjust to the toplevel to get the coords right, as setting
+     * the IME composition window is based on the toplevel hwnd, so
+     * ignore height.
+     */
+
+    while (!Tk_IsTopLevel(tkwin)) {
+	x += Tk_X(tkwin);
+	y += Tk_Y(tkwin);
+	tkwin = Tk_Parent(tkwin);
+	if (tkwin == NULL) {
+	    return;
+	}
+    }
+
+    win = Tk_WindowId(tkwin);
+    if (win) {
+	HIMC hIMC;
+	HWND hwnd = Tk_GetHWND(win);
+
+	if (hwnd != caretHWND) {
+	    DestroyCaret();
+	    if (CreateCaret(hwnd, NULL, 0, 0)) {
+		caretHWND = hwnd;
+	    }
+	}
+
+	if (!SetCaretPos(x, y) && CreateCaret(hwnd, NULL, 0, 0)) {
+	    caretHWND = hwnd;
+	    SetCaretPos(x, y);
+	}
+
+	/*
+	 * The IME composition window should be updated whenever the caret
+	 * position is changed because a clause of the composition string may
+	 * be converted to the final characters and the other clauses still
+	 * stay on the composition window.  -- yamamoto
+	 */
+	hIMC = ImmGetContext(hwnd);
+	if (hIMC) {
+	    COMPOSITIONFORM cform;
+	    cform.dwStyle = CFS_POINT;
+	    cform.ptCurrentPos.x = x;
+	    cform.ptCurrentPos.y = y;
+	    ImmSetCompositionWindow(hIMC, &cform);
+	    ImmReleaseContext(hwnd, hIMC);
+	}
+    }
+}
